#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <cstring>
#include <cstdio>
#include <vector>
#include <mysql/mysql.h>
#include <pthread.h>
#include <errno.h>
#include <time.h>

extern int errno;

using namespace std;

static const char* host= "localhost";
const char* user= "root";
const char* passwd= "202409";
const char* database = "tencent";
const int port = 3306;
const char* unix_socket ="/tmp/mysql.sock";
char gender[4] ={"NMF"};

class MySql;
class QueryThread;
typedef bool (MySql::*ForEachTableFn)(int, int, string&);
typedef vector<string> StringVector;
typedef vector<QueryThread*> QueryThreadVector;

void* threadProc(void* arg);


class QueryThread
{
public:
    pthread_t thread;
    MYSQL db;
    string queryStr;
    bool isConnected;
    bool isRunning;
    QueryThread():isConnected(false), isRunning(false)
    {
    }
    ~QueryThread()
    {
        if (isConnected)
        {

            mysql_close(&db);
        }
    }

    const char* getLastError()
    {
        if (isConnected)
        {
            return mysql_error(&db);
        }
        else
        {
            return "mysql is not connected yet!";
        }
    }
    bool connect()
    {
        if (isConnected)
        {
            mysql_close(&db);
            isConnected = false;
            //return true;
        }
        if (mysql_init(&db)!= NULL)
        {
            char value = 1;
            unsigned int timeOut = 28800;
            if (mysql_options(&db, MYSQL_OPT_LOCAL_INFILE, &value) == 0
                &&mysql_options(&db, MYSQL_OPT_RECONNECT, &value) == 0
                &&mysql_options(&db, MYSQL_OPT_CONNECT_TIMEOUT, &value) == 0
                &&mysql_options(&db, MYSQL_OPT_READ_TIMEOUT, &timeOut) == 0
                &&mysql_options(&db, MYSQL_OPT_WRITE_TIMEOUT, &timeOut) == 0

                )
            {
                if (mysql_real_connect(&db, host, user, passwd, database, port, unix_socket,
                                       CLIENT_MULTI_STATEMENTS|CLIENT_LOCAL_FILES) != NULL)
                {
                    if (mysql_set_server_option(&db, MYSQL_OPTION_MULTI_STATEMENTS_ON) == 0)
                    {
                        cout << "connection successful!" << endl;
                        isConnected = true;
                    }

                }
            }
        }


        if (!isConnected)
        {
            cout << "connect failed error:" << mysql_error(&db) << endl;
            cout.flush();
        }
        return isConnected;
    }

    bool exec(const string& queryString)
    {
        queryStr = queryString;
        // a safe check
        if (mysql_thread_safe() != 1)
        {
            cout << "*********************could this happen?*******************"<<endl;
            return false;
        }

        if (pthread_create(&thread, NULL, threadProc, this) == 0)
        {
            isRunning = true;
        }


        return isRunning;
    }
    bool tryJoin()
    {
        if (isRunning)
        {
            void* ptr = NULL;
            if (pthread_tryjoin_np(thread, &ptr) == 0)
            {
                isRunning = false;
            }
        }
        return !isRunning;
    }
    bool join()
    {
        if (isRunning)
        {
            void* ptr = NULL;
            if (pthread_join(thread, &ptr) == 0)
            {
                isRunning = false;
            }
        }
        return !isRunning;
    }
    bool createSchema()
    {
        bool ret = false;
        if ((mysql_init(&db))!= NULL)
        {
            if (mysql_real_connect(&db, host, user, passwd, NULL, port, unix_socket, 0) != NULL)
            {
                string str = "CREATE SCHEMA IF NOT EXISTS ";
                str += database;

                if (mysql_real_query(&db,  str.c_str(), str.size())!= 0)
                {
                    ret = true;
                }
                mysql_close(&db);
            }
        }
        return ret;
    }
};

class MySql
{
public:

    MySql()
    {
        mysql_library_init(0, NULL, NULL);

    }
    ~MySql()
    {

        mysql_library_end();
    }
    bool createSchema()
    {
        QueryThread query;
	    return query.createSchema();
    }

    bool collectAllTableNamePostfix(StringVector& strVect);
    bool execQuery(const string& str);
    bool execQuery(const StringVector& strVect);
	bool execAllYearGenderQuery(ForEachTableFn fn);
	bool execAllYearGenderQueryString(ForEachTableFn fn);
	bool createYearGenderTables();
	bool createYearGenderRecResultTables(int i, int j, string& str);
	bool insertYearGenderRecResultTables(int i, int j, string& str);
	bool createUserKeywordYearGenderTables(int i, int j, string& str);

	// leader board year-gender
	bool createYearGenderLeaderBoardTables(int i, int j, string& str);

	// final evaluation year-gender
	bool createYearGenderFinalEvaluationTables(int i, int j, string& str);


	bool insertUserKeywordYearGenderTables(int i, int j, string& str);
	bool createYearGenderItemKeywordTables(int i, int j, string& str);
    bool insertYearGenderItemKeywordTables(int i, int j, string& str);

	bool updateTagCountInYearGenderTables(int i, int j, string& str);
	bool addTagCountInYearGenderTables(int i, int j, string& str);
	bool createYearGenderOriginalRecResultTables(int i, int j, string& str);
	bool insertYearGenderOriginalRecResultTables(int i, int j, string& str);
	bool createAllTables();
	bool createAlgorithmUserItemKeyword(int i, int j, string& str);
	bool insertAlgorithmUserItemKeyword(int i, int j, string& str);
	bool createYearGenderUserItem(int i, int j, string& str);
	bool insertYearGenderUserItem(int i, int j, string& str);


    bool createYearGenderUserSnsUserItemTables(int i, int j, string& str);

    bool createYearGenderUserSnsFollowerTables(int i, int j, string& str);
	bool evaluateAlgorithmUserItemKeyword(int i, int j, string& str);
	bool exportResult(const string& strTableName, const string& strFileName);
	bool trainingAlgorithmUserItemKeyword(int i, int j, string& str);
	bool removeTables(int i, int j, string& str);
	bool modify(int i, int j, string& str);
	bool query();
	int getTableCount(const char* tableName);
    static const char* statements[];
protected:
	char queryStr[2048];
};

const char* MySql::statements[]=
{
    // user_profile
    "drop table if exists user_profile;"
    "create table user_profile(UserId integer primary key, YearOfBirth integer not null, "
            "Gender integer not null, NumberOfTweet integer not null, TagIds text not null);"
    "load data local infile '/home/nick/download/track1/user_profile.txt' into table user_profile;"
    "alter table user_profile add index user_profile_UserId_index(UserId);",

    // user_profile_detail
    "drop table if exists user_profile_detail;"
    "create table user_profile_detail(UserId integer not null, TagId integer not null, primary key(UserId, TagId));"
    "load data local infile '/home/nick/download/track1/user_profile_detail.txt' into table user_profile_detail;"
    "alter table user_profile_detail add index user_profile_detail_UserId_index(UserId);"
    "alter table user_profile_detail add index user_profile_detail_TagId_index(TagId);",

    // item_detail
    "drop table if exists item_detail;"
    "create table item_detail(ItemId integer not null, Keyword integer not null);"
    "load data local infile '/home/nick/download/track1/item_detail.txt' into table item_detail;"
    "alter table item_detail add index item_detail_Item_index(ItemId);"
    "alter table item_detail add index item_detail_Keyword_index(Keyword);" // no comma! should be same statement
    //item_detail_weight
    "drop table if exists item_detail_weight;"
    "create table item_detail_weight(ItemId integer not null, Keyword integer not null, Weight real not null, "
            "primary key(ItemId, Keyword));"
    "insert into item_detail_weight(itemid, keyword, weight)select itemid, keyword, "
        "count(*) as weight from item_detail group by itemid, keyword;"
    "update item_detail_weight w left join (select itemid, count(*) as count from item_detail "
        " group by itemid) i on w.itemid=i.itemid set w.weight=w.weight/i.count;"

    "alter table item_detail_weight add index item_detail_weight_ItemId_index(ItemId);"
    "alter table item_detail_weight add index item_detail_weight_Keyword_index(Keyword);"
    "alter table item_detail_weight add index item_detail_weight_Weight_index(Weight);",

    // user_keyword_detail
    "drop table if exists user_keyword_detail;"
    "create table user_keyword_detail(UserId integer not null, Keyword integer not null, Weight real not null, "
            "primary key(UserId, keyword));"
    "load data local infile '/home/nick/download/track1/user_key_word_detail.txt' into table user_keyword_detail;"
    "alter table user_keyword_detail add index user_keyword_detail_Keyword_index(Keyword);"
    "alter table user_keyword_detail add index user_keyword_detail_UserId_index(UserId);",

    // rec_log_train
    "drop table if exists rec_log_train;"
    "create table rec_log_train(UserId integer not null, ItemId integer not null, Result integer"
        " not null, UnixTimestamp integer not null);"
    "load data local infile '/home/nick/download/track1/rec_log_train.txt' into table rec_log_train;"

    "alter table rec_log_train add index rec_log_train_UserId_index(UserId);"
    "alter table rec_log_train add index rec_log_train_UnixTimestamp_index(UnixTimestamp);"
    "alter table rec_log_train add index rec_log_train_ItemId_index(ItemId);",

    // leaderboard
    "drop table if exists rec_log_test;"
    "create table rec_log_test(UserId integer not null, ItemId integer not null, Result integer"
        " not null, UnixTimestamp integer not null);"
    "load data local infile '/home/nick/download/track1/rec_log_test.txt' into table rec_log_test;"
    "alter table rec_log_test add index rec_log_test_UserId_index(UserId);"
    "alter table rec_log_test add index rec_log_test_UnixTimestamp_index(UnixTimestamp);"
    "alter table rec_log_test add index rec_log_test_ItemId_index(ItemId);"

    //leader_board
    "drop table if exists leader_board;"
    "create table leader_board(UserId integer not null, ItemId integer not null, Result integer"
        " not null, UnixTimestamp integer not null);"
    "insert into leader_board(UserId,ItemId,Result,UnixTimestamp)select UserId,ItemId,Result,"
        "UnixTimestamp from rec_log_test where UnixTimestamp<1321891200;"
    "alter table leader_board add index leader_board_UserId_index(UserId);"
    "alter table leader_board add index leader_board_UnixTimestamp_index(UnixTimestamp);"
    "alter table leader_board add index leader_board_ItemId_index(ItemId);"
    // final_evaluation
    "drop table if exists final_evaluation;"
    "create table final_evaluation(UserId integer not null, ItemId integer not null, Result integer"
        " not null, UnixTimestamp integer not null);"
    "insert into final_evaluation(UserId,ItemId,Result,UnixTimestamp)select UserId,ItemId,Result,"
        "UnixTimestamp from rec_log_test where UnixTimestamp>=1321891200;"
    "alter table final_evaluation add index final_evaluation_UserId_index(UserId);"
    "alter table final_evaluation add index final_evaluation_UnixTimestamp_index(UnixTimestamp);"
    "alter table final_evaluation add index final_evaluation_ItemId_index(ItemId);",

    // item_category
    "drop table if exists  item_category;"
    "create table item_category (ItemId integer NOT NULL, Category1 integer NOT NULL, Category2 integer NOT NULL, "
            "Category3 integer NOT NULL, Category4 integer NOT NULL, primary key(Itemid));"
    "load data local infile '/home/nick/download/track1/item_category.txt' into table item_category;"
    "alter table item_category add index item_category_Category1_index(Category1);"
    "alter table item_category add index item_category_Category2_index(Category2);"
    "alter table item_category add index item_category_Category3_index(Category3);"
    "alter table item_category add index item_category_Category4_index(Category4);",
    // user_sns
    "drop table if exists user_sns;"
    "create table user_sns(FollowerId integer not null, FolloweeId integer not null);"
    "load data local infile '/home/nick/download/track1/user_sns.txt' into table user_sns;"
    "alter table user_sns add primary key(FollowerId, FolloweeId);"
    "alter table user_sns add index user_sns_FollowerId_index(FollowerId);"
    "alter table user_sns add index user_sns_FolloweeId_index(FolloweeId);",
    // user_item_sns
    "drop table if exists user_item_sns;"
    "create table if not exists user_item_sns(UserId integer not null, ItemId integer not null,"
         "Weight real not null, primary key(UserId, ItemId));"
    "alter table user_item_sns add index user_item_sns_UserId_index(UserId);"
    "alter table user_item_sns add index user_item_sns_ItemId_index(ItemId);"
    "alter table user_item_sns add index user_item_snss_Weight_index(Weight);",

    "CREATE TABLE item_item_keyword(SrcItemId integer NOT NULL, DstItemId integer NOT NULL, "
        "Keyword integer NOT NULL, primary key(SrcItemId,DstItemId,Keyword));"
    "alter table item_item_keyword add index item_item_keyword_SrcItemId_index(SrcItemId);"
    "alter table item_item_keyword add index item_item_keyword_DstItemId_index(DstItemId);"
    "alter table item_item_keyword add index item_item_keyword_Keyword_index(Keyword);"
    "insert into item_item_keyword(SrcItemId, DstItemId,Keyword) select l.ItemId, r.ItemId, l.Keyword "
        "from item_detail l, item_detail r where l.ItemId<>r.ItemId and l.Keyword=r.Keyword;",
    "CREATE TABLE item_item_relation(SrcItemId integer NOT NULL, DstItemId integer NOT NULL,"
        "Relation float not NULL);"
    "alter table item_item_relation add index item_item_relation_SrcItemId_index(SrcItemId);"
    "alter table item_item_relation add index item_item_relation_DstItemId_index(DstItemId);"
    "insert into item_item_relation(SrcItemId,DstItemId,Relation)select SrcItemId,DstItemId,"
        "count(*)*count(*) as Relation from item_item_keyword group by SrcItemId,DstItemId "
        " having count(*)>1;",

};

int MySql::getTableCount(const char* tableName)
{
    MYSQL db;
    int count = -1;
    char queryBuf[1024];
	if (mysql_init(&db)!= NULL)
    {
        char value = 1;
        unsigned int timeOut = 28800;
        if (mysql_options(&db, MYSQL_OPT_LOCAL_INFILE, &value) == 0
            &&mysql_options(&db, MYSQL_OPT_RECONNECT, &value) == 0
            &&mysql_options(&db, MYSQL_OPT_CONNECT_TIMEOUT, &value) == 0
            &&mysql_options(&db, MYSQL_OPT_READ_TIMEOUT, &timeOut) == 0
            &&mysql_options(&db, MYSQL_OPT_WRITE_TIMEOUT, &timeOut) == 0

            )
        {
            if (mysql_real_connect(&db, host, user, passwd, database, port, unix_socket,
                                       CLIENT_MULTI_STATEMENTS|CLIENT_LOCAL_FILES) != NULL)
            {
                if (mysql_set_server_option(&db, MYSQL_OPTION_MULTI_STATEMENTS_ON) == 0)
                {
                    sprintf(queryBuf, "select count(*) from %s;", tableName);

                    if (mysql_real_query(&db, queryBuf, strlen(queryBuf))== 0)
                    {
                        MYSQL_RES * res = mysql_use_result(&db);
                        if (res)
                        {
                            unsigned int num_fields = mysql_num_fields(res);
                            if (num_fields == 1)
                            {
                                MYSQL_ROW row = mysql_fetch_row(res);
                                if (row)
                                {
                                    sscanf(row[0], "%d", &count);
                                    cout << "select count(*) from table " << tableName << " is " << count <<endl;
                                }
                            }
                            mysql_free_result(res);
                        }
                    }
                }
                mysql_close(&db);
            }
        }
    }
	return count;
}



bool MySql::exportResult(const string& strTableName, const string& strFileName)
{
    return true;
}


bool MySql::trainingAlgorithmUserItemKeyword(int i, int j, string& str)
{
	char dstTableName[128];
	char srcTableName[128];
	const char* algorithm = "AlgorithmUserItemKeyword";

	// LeaderBoard
	sprintf(dstTableName, "YearGenderOriginalRecResult%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderUserItem%d%c", i, gender[j]);


    //sprintf(queryStr, "alter table %s drop column %s;", dstTableName, algorithm);
	//str = queryStr;

    sprintf(queryStr, "alter table %s add column (%s integer default 0);", dstTableName, algorithm);
	str += queryStr;
	sprintf(queryStr, "update %s dst, %s src set dst.%s=1 where dst.UserId=src.UserId and "
         "dst.ItemId=src.ItemId;", dstTableName, srcTableName, algorithm);
	str += queryStr;

	return true;
}

bool MySql::evaluateAlgorithmUserItemKeyword(int i, int j, string& str)
{
	char dstTableName[128];
	char srcTableName[128];
	const char* algorithm = "AlgorithmUserItemKeyword";

	// LeaderBoard
	sprintf(dstTableName, "YearGenderLeaderBoard%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderUserItem%d%c", i, gender[j]);


    //sprintf(queryStr, "alter table %s drop column %s;", dstTableName, algorithm);
	//str = queryStr;

    sprintf(queryStr, "alter table %s add column (%s integer default 0);", dstTableName, algorithm);
	str += queryStr;
	sprintf(queryStr, "update %s dst, %s src set dst.%s=1 where dst.UserId=src.UserId and "
         "dst.ItemId=src.ItemId;", dstTableName, srcTableName, algorithm);
	str += queryStr;

	// finalEvaluation
	sprintf(dstTableName, "YearGenderFinalEvaluation%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderUserItem%d%c", i, gender[j]);

    sprintf(queryStr, "alter table %s drop column %s;", dstTableName, algorithm);
	str += queryStr;

    sprintf(queryStr, "alter table %s add column (%s integer default 0);", dstTableName, algorithm);
	str += queryStr;
	sprintf(queryStr, "update %s dst, %s src set dst.%s=1 where dst.UserId=src.UserId and "
         "dst.ItemId=src.ItemId;", dstTableName, srcTableName, algorithm);
	str += queryStr;

	return true;
}

bool MySql::createAllTables()
{
	bool ret = true;
	unsigned int i;

    StringVector strVect;
	for (i = 0; i < sizeof(statements)/sizeof(char*); i ++)
	{
	    strVect.push_back(statements[i]);
	}
    if (!execQuery(strVect))
    {
        cout << "exec failed" <<endl;
        ret = false;
    }

	return ret;
}


bool MySql::execQuery(const string& str)
{
    QueryThread query;
    if (query.exec(str))
    {
        if (query.join())
        {
            return true;
        }
    }
    else
    {
        cout << "query '" << str << "' failed of " << query.getLastError() << endl;
    }
    return false;
}

bool MySql::execQuery(const StringVector& strVect)
{
    size_t i;
    bool ret = true;
    QueryThreadVector queryVect;
    for (i = 0; i < strVect.size(); i ++)
    {
        QueryThread* query = new QueryThread;
        if (query && query->exec(strVect[i]))
        {
            queryVect.push_back(query);
        }
        else
        {
            ret = false;
            delete query;
        }
    }

#ifndef NICK_BLOCKING
    while (queryVect.size() > 0)
    {
        sleep(5);
        for (QueryThreadVector::iterator it= queryVect.begin(); it != queryVect.end(); it ++)
        {
            if ((*it)->tryJoin())
            {
                delete *it;
                queryVect.erase(it);
                break;
            }
        }
    }
#else
    for (i = 0; i < queryVect.size(); i ++)
    {
        if (!queryVect[i]->join())
        {
            ret = false;
        }
        delete queryVect[i];
    }


#endif
    return ret;
}

bool MySql::collectAllTableNamePostfix(StringVector& strVect)
{
    int i;
    char buf[256];
	for (i = 2000; i >= 1940; i --)
	{
		if (i < 1980 && i%10 != 0)
		{
			continue;
		}
		for (int j = 0; j < 3; j ++)
		{
		    sprintf(buf, "%d%c", i, gender[j]);

			strVect.push_back(buf);
		}
	}
	return true;
}

bool MySql::execAllYearGenderQuery(ForEachTableFn fn)
{
	StringVector strVect;
	int i;
	for (i = 2000; i >= 1940; i --)
	{
		if (i < 1980 && i%10 != 0)
		{
			continue;
		}
		for (int j = 0; j < 3; j ++)
		{
		    string str;
			(this->*fn)(i, j, str);

			strVect.push_back(str);
		}
	}

	return execQuery(strVect);
}

bool MySql::execAllYearGenderQueryString(ForEachTableFn fn)
{
	int i;
	string query;
	for (i = 2000; i >= 1940; i --)
	{
		if (i < 1980 && i%10 != 0)
		{
			continue;
		}
		for (int j = 0; j < 3; j ++)
		{
		    string str;
			(this->*fn)(i, j, str);

			query += str;
		}
	}

	return execQuery(query);
}

bool MySql::createYearGenderTables()
{
    StringVector strVect;
	char condition[256];
	for (int i = 2000; i >= 1940; i --)
	{
		if (i < 1980 && i%10 != 0)
		{
			continue;
		}
		for (int j = 0; j < 3; j ++)
		{
			char tableName[128];
			sprintf(tableName, "YearGender%d%c", i, gender[j]);
            sprintf(queryStr, "drop table if exists %s;", tableName);
            string str = queryStr;
			sprintf(queryStr, "create table %s(UserId integer primary key);", tableName);
			str += queryStr;
			if (i == 2000)
			{
				sprintf(condition, "YearOfBirth>=2000");
			}
			if (i == 1940)
			{
				sprintf(condition, "YearOfBirth<1950");
			}
			if (i >= 1980 && i < 2000)
			{
				sprintf(condition, "YearOfBirth=%d", i);
			}
			if (i > 1940 && i < 1980)
			{
				sprintf(condition, "YearOfBirth<%d and YearOfBirth>=%d", i+10, i);
			}
			sprintf(queryStr, "insert into %s(UserId) select UserId from user_profile where Gender=%d and %s;", tableName, j, condition);
			str += queryStr;
			strVect.push_back(str);
		}
	}
	return execQuery(strVect);
}


bool MySql::addTagCountInYearGenderTables(int i, int j, string& str)
{
    char tableName[128];
    sprintf(tableName, "YearGender%d%c", i, gender[j]);

    sprintf(queryStr, "alter table %s add column TagCount integer;", tableName);
    str = queryStr;

	return true;
}

bool MySql::updateTagCountInYearGenderTables(int i, int j, string& str)
{
    char tableName[128];
    sprintf(tableName, "YearGender%d%c", i, gender[j]);

    sprintf(queryStr, "update %s set TagCount=(select count(*) as TagCount from user_profile_detail u where u.UserId=%s.UserId group by u.UserId);",
            tableName, tableName);
    str = queryStr;
	return true;
}


bool MySql::createYearGenderRecResultTables(int i, int j, string& str)
{
    char dstTableName[128];
    char srcTableName[128];
    sprintf(dstTableName, "YearGenderRecResult%d%c", i, gender[j]);
    sprintf(srcTableName, "YearGender%d%c", i, gender[j]);
    sprintf(queryStr, "drop table if exists  %s;", dstTableName);
	str = queryStr;
    sprintf(queryStr, "create table %s(UserId integer not null, ItemId integer not null,"
        " UnixTimestamp integer not null, primary key(UserId, ItemId, UnixTimestamp));", dstTableName);

    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
    str += queryStr;
	return true;
}

bool MySql::insertYearGenderRecResultTables(int i, int j, string& str)
{
    char dstTableName[128];
    char srcTableName[128];
    sprintf(dstTableName, "YearGenderRecResult%d%c", i, gender[j]);
    sprintf(srcTableName, "YearGender%d%c", i, gender[j]);


    sprintf(queryStr, "insert into %s(UserId, ItemId, UnixTimestamp) select t.UserId, t.ItemId, t.UnixTimestamp"\
        " from rec_log_train t, %s s where t.Result=1 and t.UserId=s.UserId;", dstTableName, srcTableName);
    str = queryStr;

	return true;
}

bool MySql::createYearGenderLeaderBoardTables(int i, int j, string& str)
{
    char dstTableName[128];
    char srcTableName[128];
    sprintf(dstTableName, "YearGenderLeaderBoard%d%c", i, gender[j]);
    sprintf(srcTableName, "YearGender%d%c", i, gender[j]);
    sprintf(queryStr, "drop table if exists %s;", dstTableName);
	str = queryStr;
    sprintf(queryStr, "create table %s(UserId integer not null, ItemId integer not null,"
        " UnixTimestamp integer not null, primary key(UserId, ItemId, UnixTimestamp));", dstTableName);
    str += queryStr;
    sprintf(queryStr, "insert into %s(UserId, ItemId, UnixTimestamp) select t.UserId, t.ItemId, t.UnixTimestamp"\
        " from leader_board t, %s s where t.UserId=s.UserId;", dstTableName, srcTableName);

    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_UnixTimestamp_index(UnixTimestamp);", dstTableName, dstTableName);
    str += queryStr;
	return true;

}

bool MySql::createYearGenderFinalEvaluationTables(int i, int j, string& str)
{
    char dstTableName[128];
    char srcTableName[128];
    sprintf(dstTableName, "YearGenderFinalEvaluation%d%c", i, gender[j]);
    sprintf(srcTableName, "YearGender%d%c", i, gender[j]);
    sprintf(queryStr, "drop table if exists  %s;", dstTableName);
	str = queryStr;
    sprintf(queryStr, "create table %s(UserId integer not null, ItemId integer not null,"
        " UnixTimestamp integer not null, primary key(UserId, ItemId, UnixTimestamp));", dstTableName);
    str += queryStr;
    sprintf(queryStr, "insert into %s(UserId, ItemId, UnixTimestamp) select t.UserId, t.ItemId, t.UnixTimestamp"\
        " from leader_board t, %s s where t.UserId=s.UserId;", dstTableName, srcTableName);
    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_UnixTimestamp_index(UnixTimestamp);", dstTableName, dstTableName);
    str += queryStr;
	return true;

}


bool MySql::createUserKeywordYearGenderTables(int i, int j, string& str)
{
    char dstTableName[128];
    char srcTableName[128];
    sprintf(dstTableName, "UserKeywordYearGender%d%c", i, gender[j]);
    sprintf(srcTableName, "YearGender%d%c", i, gender[j]);

    sprintf(queryStr, "drop table if exists %s;", dstTableName);
    str = queryStr;

    sprintf(queryStr, "create table %s(UserId integer not null, Keyword integer not null, Weight real not null, primary key(UserId, Keyword));", dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_Keyword_index(Keyword);", dstTableName, dstTableName);
    str += queryStr;
	return true;
}


bool MySql::insertUserKeywordYearGenderTables(int i, int j, string& str)
{
    char dstTableName[128];
    char srcTableName[128];
    sprintf(dstTableName, "UserKeywordYearGender%d%c", i, gender[j]);
    sprintf(srcTableName, "YearGender%d%c", i, gender[j]);


    sprintf(queryStr, "insert into %s(UserId, Keyword, Weight) select s.UserId, Keyword, Weight from user_keyword_detail s, %s t where s.UserId=t.UserId;",
            dstTableName, srcTableName);
    str = queryStr;

	return true;
}


bool MySql::createYearGenderItemKeywordTables(int i, int j, string& str)
{
	char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "YearGenderItemKeyword%d%c", i, gender[j]);
	sprintf(srcTableName, "UserKeywordYearGender%d%c", i, gender[j]);
	sprintf(queryStr, "drop table if exists  %s;", dstTableName);
	str = queryStr;
	sprintf(queryStr, "create table %s(UserId integer not null, ItemId integer not null, Keyword "
         "integer not null, Weight real not null, primary key(UserId, ItemId, Keyword));",
         dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_Keyword_index(Keyword);", dstTableName, dstTableName);
	str += queryStr;
	return true;
}


bool MySql::insertYearGenderItemKeywordTables(int i, int j, string& str)
{
	char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "YearGenderItemKeyword%d%c", i, gender[j]);
	sprintf(srcTableName, "UserKeywordYearGender%d%c", i, gender[j]);

	sprintf(queryStr, "insert into %s(UserId, ItemId, Keyword, Weight) select s.UserId, i.ItemId, "
         "s.Keyword, s.Weight*i.Weight as Weight from %s s, item_detail_weight i where s.Keyword=i.Keyword;",
			dstTableName, srcTableName);
	str = queryStr;
	return true;
}

bool MySql::createYearGenderOriginalRecResultTables(int i, int j, string& str)
{
    char dstTableName[128];
    char srcTableName[128];
    sprintf(dstTableName, "YearGenderOriginalRecResult%d%c", i, gender[j]);
    sprintf(srcTableName, "YearGender%d%c", i, gender[j]);

    sprintf(queryStr, "drop table if exists  %s;", dstTableName);
	str = queryStr;
    sprintf(queryStr, "create table %s(UserId integer not null, ItemId integer not null, Result integer not null,UnixTimestamp integer not null);", dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_UnixTimestamp_index(UnixTimestamp);", dstTableName, dstTableName);
    str += queryStr;

	return true;
}

bool MySql::insertYearGenderOriginalRecResultTables(int i, int j, string& str)
{
    char dstTableName[128];
    char srcTableName[128];
    sprintf(dstTableName, "YearGenderOriginalRecResult%d%c", i, gender[j]);
    sprintf(srcTableName, "YearGender%d%c", i, gender[j]);

    sprintf(queryStr, "insert into %s(UserId, ItemId, Result, UnixTimestamp) select s.UserId, "
            "s.ItemId, s.Result, s.UnixTimestamp from %s y, rec_log_train s where s.UserId=y.UserId;", dstTableName, srcTableName);
    str = queryStr;

	return true;
}


bool MySql::createYearGenderUserSnsUserItemTables(int i, int j, string& str)
{
    char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "YearGenderUserSnsUserItem%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderUserSnsFollower%d%c", i, gender[j]);

    StringVector strVect;
    collectAllTableNamePostfix(strVect);

    sprintf(queryStr, "drop table if exists %s;", dstTableName);
    str += queryStr;

    sprintf(queryStr, "create table if not exists %s(UserId integer not null, ItemId integer"
            " not null, Weight real not null, primary key(UserId, ItemId));", dstTableName);
    str += queryStr;
    sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_Weight_index(Weight);", dstTableName, dstTableName);
    str += queryStr;

    for (int num = 0; num < strVect.size(); num ++)
    {
        char tableName[256];
        sprintf(tableName, "YearGenderUserItem%s", strVect[num].c_str());
        sprintf(queryStr, "insert into %s(UserId, ItemId, Weight)select src.UserId, "
                "src.ItemId, src.Weight from %s src, %s f where src.UserId=f.FolloweeId ON DUPLICATE "
                "KEY UPDATE %s.Weight=%s.Weight+src.Weight;", dstTableName, tableName, srcTableName,
                dstTableName, dstTableName);
        str += queryStr;

    }
	return true;
}

bool MySql::modify(int i, int j, string& str)
{
    char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "YearGenderUserSnsUserItem%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderUserSnsFollower%d%c", i, gender[j]);


	sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_Weight_index(Weight);", dstTableName, dstTableName);
    str += queryStr;
	return true;
}

bool MySql::query()
{
    char dstTableName[128];
	char srcTableName[128];

    StringVector strVect;
    collectAllTableNamePostfix(strVect);
    for (int i = 0; i < strVect.size(); i ++)
    {
        sprintf(dstTableName, "YearGenderUserSnsUserItem%s", strVect[i].c_str());
        sprintf(srcTableName, "YearGenderUserSnsFollower%s", strVect[i].c_str());

        cout << dstTableName << ":" << getTableCount(dstTableName) << endl;
        cout << srcTableName << ":" << getTableCount(srcTableName) << endl;
    }
	return true;
}


bool MySql::createYearGenderUserSnsFollowerTables(int i, int j, string& str)
{
    char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "YearGenderUserSnsFollower%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGender%d%c", i, gender[j]);

    sprintf(queryStr, "drop table if exists %s;", dstTableName);
    str += queryStr;

    sprintf(queryStr, "create table %s(FollowerId integer not null, FolloweeId integer"
            " not null, primary key(FollowerId, FolloweeId));", dstTableName);
    str += queryStr;

	sprintf(queryStr, "insert into %s(FollowerId, FolloweeId)select FollowerId, FolloweeId "
         " from user_sns s, %s u where u.UserId=s.FollowerId;", dstTableName, srcTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_FollowerId_index(FollowerId);", dstTableName, dstTableName);
    str += queryStr;

    sprintf(queryStr, "alter table %s add index %s_FolloweeId_index(FolloweeId);", dstTableName, dstTableName);
    str += queryStr;

	return true;
}


bool MySql::createYearGenderUserItem(int i, int j, string& str)
{
	char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "YearGenderUserItem%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderItemKeyword%d%c", i, gender[j]);

    sprintf(queryStr, "drop table if exists  %s;", dstTableName);
	str = queryStr;
	sprintf(queryStr, "create table %s(UserId integer not null, ItemId integer not null, Weight real not null, primary key(UserId, ItemId));", dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_Weight_index(Weight);", dstTableName, dstTableName);
	str += queryStr;
	return true;
}

bool MySql::insertYearGenderUserItem(int i, int j, string& str)
{
	char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "YearGenderUserItem%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderItemKeyword%d%c", i, gender[j]);

 	sprintf(queryStr, "insert into %s(UserId, ItemId, Weight) select UserId, ItemId, sum(Weight) "
          "as Weight from %s group by UserId, ItemId order by UserId, ItemId;", dstTableName, srcTableName);
	str = queryStr;

	return true;
}



bool MySql::createAlgorithmUserItemKeyword(int i, int j, string& str)
{
	char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "AlgorithmUserItemKeywordYearGender%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderUserItem%d%c", i, gender[j]);


    sprintf(queryStr, "drop table if exists  %s;", dstTableName);
	str = queryStr;

	sprintf(queryStr, "create table %s(UserId integer not null, ItemId integer not null, "
         "Weight real not null, Rank integer not null, primary key(UserId, ItemId));", dstTableName);
	str += queryStr;

	sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
	str += queryStr;

	sprintf(queryStr, "alter table %s add index %s_Weight_index(Weight);", dstTableName, dstTableName);
	str += queryStr;

	return true;
}


bool MySql::insertAlgorithmUserItemKeyword(int i, int j, string& str)
{
	char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "AlgorithmUserItemKeywordYearGender%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderUserItem%d%c", i, gender[j]);

    // step 1: let's just concentrate on rank without update weight.
    // we will pick up rank 1 for those having at least one item, so does rank2,3
    for (int rank = 1; rank <= 3; rank ++)
    {
        sprintf(queryStr, "insert into %s(UserId, ItemId, Rank) select UserId, "
            "cast("
                "substring_index("
                    "SUBSTRING_INDEX(GROUP_CONCAT(ItemId order by Weight desc, ItemId asc),"
                 " ',', %d), ',', -1)"
             "as unsigned) as ItemId,"
             " %d as Rank from %s group by userid having count(*)>= %d;",
             dstTableName, rank, rank, srcTableName, rank);
        str += queryStr;
    }

    // step 2: now let's update the weight for all records.
    sprintf(queryStr, "update %s t, %s s set t.Weight=s.Weight where t.UserId=s.UserId and "
            "t.ItemId=s.ItemId;",dstTableName, srcTableName);
    str += queryStr;
	return true;
}

bool MySql::removeTables(int i, int j, string& str)
{
    char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "UserItemKeywordRankYearGender%d%c", i, gender[j]);
	sprintf(srcTableName, "YearGenderUserItem%d%c", i, gender[j]);
	sprintf(queryStr, "drop table if exists %s;", dstTableName);
	str = queryStr;

	return true;
}

#ifndef NICK_TEST
void* threadProc(void* arg)
{
    QueryThread* thread = (QueryThread*) arg;
    if (thread->queryStr.size() == 0)
    {
        cout<< "this should never happen!" <<endl;
        return NULL;
    }

    size_t startPos = 0, currentPos = 0;
    do
    {
        currentPos = thread->queryStr.find_first_of(';', startPos);
        string str = thread->queryStr.substr(startPos, currentPos-startPos);
        if (str.size() == 0)
        {
            cout << endl<<"***"<<endl<<"this is the end of query: "<<thread->queryStr <<endl<<"*****"<<endl;
            break;
        }
        str += ";";
        if (thread->connect())
        {
            int err = 0;
            do
            {
                //let's wait to see if transaction can finish before proceed because it will
                // create more connections before this transaction finishes...
                if ((err = mysql_real_query(&(thread->db), "START TRANSACTION;", strlen("START TRANSACTION;")))!= 0)
                {
                    cout << "cannot start transaction! : " << mysql_error(&(thread->db))<<endl;
                    break;
                }
                if ((err = mysql_real_query(&(thread->db), str.c_str(), str.size()))!= 0)
                {
                    if ((err = mysql_real_query(&(thread->db), "ROLLBACK;", strlen("ROLLBACK;")))!= 0)
                    {
                        cout << "cannot roll back! : " << mysql_error(&(thread->db)) << endl;
                        break;
                    }
                    if (mysql_errno(&(thread->db)) == 1205)
                    {
                        cout << endl<< "**************error of 1205**************" << endl;
                        cout << endl << "query:" << str << endl;
                        // will retry, don't break;
                        //sleep(10);
                    }
                    else
                    {
                        cout << "*********************exec query*********************"<<endl
                        << str << " with error:#####################" <<endl<< mysql_error(&(thread->db))
                        <<endl<< "##############################"<< endl;
                        cout<< "the original query is:" << str << endl;
                        cout.flush();
                        break;
                    }

                }
                else
                {
                    if ((err = mysql_real_query(&(thread->db), "COMMIT;", strlen("COMMIT;")))!= 0)
                    {
                        cout << "cannot commit! : " << mysql_error(&(thread->db)) << endl;
                        cout <<"query of '" << str << "' is not committed" <<endl;
                        break;
                    }
                }
            }
            while (err != 0);

            if (err == 0)
            {
                //cout<< thread->queryStr.c_str() <<endl;
                while (true)
                {
                    if (mysql_field_count(&(thread->db)) != 0 )
                    {

                        MYSQL_RES *res = mysql_store_result(&(thread->db));
                        if (res != NULL)
                        {
                            mysql_free_result(res);
                        }
                        else
                        {
                            cout << "***************ERROR!!!!:no resultset is returned when expected*************"<<endl;
                            break;
                        }
                    }
                    int status =  mysql_next_result(&(thread->db));
                    if (status == -1)
                    {
                        cout<< "eof"<<endl;
                        break;
                    }
                    else
                    {
                        if (status == 0)
                        {
                            cout << "continue" << endl;
                        }
                        else
                        {
                            if (status < 0)
                            {
                                cout << "error" << endl;
                                break;
                            }
                        }
                    }

                }
            }

            if (currentPos == string::npos)
            {
                break;
            }
            startPos = currentPos + 1;
            if (startPos >= thread->queryStr.size())
            {
                cout<< "end of query" <<endl;
                break;
            }
        }
    }
    while (true);

    mysql_thread_end();
    return NULL;
}
#else

void* threadProc(void* arg)
{
    QueryThread* thread = (QueryThread*) arg;
    if (thread->queryStr.size() == 0)
    {
        cout<< "this should never happen!" <<endl;
        return NULL;
    }

    string str = thread->queryStr;

    if (thread->connect())
    {
        if (mysql_real_query(&(thread->db), str.c_str(), str.size())!= 0)
        {
            cout << "*********************exec query*********************"
                << str << " with error:#####################" << mysql_error(&(thread->db))
                << "##############################"<< endl;
        }
        else
        {
            //cout<< thread->queryStr.c_str() <<endl;
            while (true)
            {
                if (mysql_field_count(&(thread->db)) != 0 )
                {
                    MYSQL_RES *res = mysql_store_result(&(thread->db));
                    if (res != NULL)
                    {
                        mysql_free_result(res);
                    }
                    else
                    {
                        cout << "***************ERROR!!!!:no resultset is returned when expected*************"<<endl;
                        break;
                    }
                }
                int status =  mysql_next_result(&(thread->db));
                if (status == -1)
                {
                    cout<< "eof"<<endl;
                    break;
                }
                else
                {
                    if (status == 0)
                    {
                        cout << "continue" << endl;
                    }
                    else
                    {
                        if (status < 0)
                        {
                            cout << "error" << endl;
                            break;
                        }
                    }
                }

            }
        }
    }

    mysql_thread_end();
    return NULL;
}
#endif


void splitItem()
{
	string inFileName = "/home/nick/download/track1/item.txt";
	string outFileName = "/home/nick/download/track1/item_detail.txt";
	ifstream inFile(inFileName.c_str());
    ofstream outFile(outFileName.c_str());
    while (true)
    {
    	int ItemId;
    	string Category, KeyWords;
    	inFile >> ItemId >> Category >> KeyWords;
    	if (inFile.eof())
    	{
    		break;
    	}
    	size_t start = 0, pos;
    	do
    	{
    		pos = KeyWords.find_first_of(';', start);
    		string keyWord = KeyWords.substr(start, pos - start);
    		//int key;
    		//sscanf(keyWord.c_str(), "%d", &key);
    		outFile << ItemId << "\t" << keyWord<< "\n";
    		start = pos + 1;
    	}
    	while (pos != string::npos);
    }
    inFile.close();
    outFile.close();
}

void splitProfile()
{
	string inFileName = "/home/nick/download/track1/user_profile.txt";
	string outFileName = "/home/nick/download/track1/user_profile_detail.txt";
	ifstream inFile(inFileName.c_str());
    ofstream outFile(outFileName.c_str());
    while (true)
    {
    	int UserId, Gender, NumberOfTweet;
    	string TagIds, YearOfBirth;
    	inFile >> UserId >> YearOfBirth >> Gender >> NumberOfTweet >> TagIds;
    	if (inFile.eof())
    	{
    		break;
    	}

    	size_t start = 0, pos;
    	do
    	{
    		pos = TagIds.find_first_of(';', start);
    		string Tag = TagIds.substr(start, pos - start);
    		outFile << UserId << "\t" << Tag << "\n";
    		start = pos + 1;
    	}
    	while (pos != string::npos);
    }
    inFile.close();
    outFile.close();
}


void splitKeyWord()
{
	string inFileName = "/home/nick/download/track1/user_key_word.txt";
	string outFileName = "/home/nick/download/track1/user_key_word_detail.txt";
	ifstream inFile(inFileName.c_str());
    ofstream outFile(outFileName.c_str());
    while (true)
    {
    	int UserId;
    	string KeyWords;
    	inFile >> UserId >> KeyWords;
    	if (inFile.eof())
    	{
    		break;
    	}
    	size_t start = 0, pos;
    	do
    	{
    		pos = KeyWords.find_first_of(';', start);
    		string keyWord = KeyWords.substr(start, pos - start);

    		string key, weight;
    		size_t splitPos = keyWord.find_first_of(':', 0);
    		key = keyWord.substr(0, splitPos);
    		weight = keyWord.substr(splitPos + 1);
    		//int key;
    		//sscanf(keyWord.c_str(), "%d", &key);
    		outFile << UserId << "\t" << key << "\t" << weight << "\n";
    		start = pos + 1;
    	}
    	while (pos != string::npos);
    }
    inFile.close();
    outFile.close();
}

void splitCategory()
{
	const string inFileName = "/home/nick/download/track1/item.txt";
    const string outFileName = "/home/nick/download/track1/item_category.txt";
    ifstream inFile(inFileName.c_str());
    ofstream outFile(outFileName.c_str());
    while (!inFile.eof()){
        int itemId;
        string category, keyword;
        inFile >> itemId >> category >> keyword;
        int subCat[4];
        size_t pos = 0, start = 0;
        int index = 0;
        while (index < 4) {
            pos = category.find_first_of('.', start);
            string sub;
            if (pos == string::npos){
                sub = category.substr(start);
            }else{
                sub = category.substr(start, pos - start);
                start = pos + 1;
            }
            sscanf(sub.c_str(), "%d", subCat + index);
            index ++;
        }
        outFile << itemId << "\t" << subCat[0] << "\t" << subCat[1] << "\t" << subCat[2]
            << "\t" << subCat[3] << "\n";
    }

}


void shrinkFile(const string& fileName, int skipped)
{
    char fileNameBuffer[1024];
    char buffer[1024];
    sprintf(fileNameBuffer, "%s.%d", fileName.c_str(), time(NULL));
    int ret = rename(fileName.c_str(), fileNameBuffer);
    if (ret == -1){
        cout << "error:" << strerror(errno)<<endl;

    }
    FILE* streamWrite = fopen(fileName.c_str(), "w");
    FILE* streamRead = fopen(fileNameBuffer, "r");
    int counter = 0;
    if (streamWrite){
        if (streamRead){
            while (fgets(buffer, 1023, streamRead)){

                if (counter % skipped == 0){
                    fputs(buffer, streamWrite);
                }
                counter ++;
            }
            fclose(streamRead);
        }
        fclose(streamWrite);
    }
}

void preprocessFiles()
{
    splitProfile();
    splitItem();
    splitKeyWord();
    splitCategory();

    shrinkFile("/home/nick/download/track1/user_profile.txt", 10);

    shrinkFile("/home/nick/download/track1/user_profile_detail.txt", 10);

    shrinkFile("/home/nick/download/track1/user_key_word.txt", 10);

    shrinkFile("/home/nick/download/track1/user_key_word_detail.txt", 10);

    shrinkFile("/home/nick/download/track1/rec_log_train.txt", 100);
}

void myFileTool()
{
    StringVector strVect;
    MySql my;
    my.collectAllTableNamePostfix(strVect);
    for (int i = 0; i < strVect.size(); i ++)
    {
        char cmd[1024];
        sprintf( cmd, "/bin/cp -f /mysqldata/mysql/tencent/YearGenderUserSnsFollower%s.frm "
            " /mysqldata/mysql/tencent/YearGenderUserSnsUserItem%s.frm",
            strVect[i].c_str(), strVect[i].c_str());

        system(cmd);
    }
}

int main()
{
    time_t startTime = time(NULL);
	MySql my;
	bool ret = false;
#ifndef NICKTEST
    //preprocessFiles();
/*
    my.createSchema();
*/
/*
    ret = my.createAllTables();

	ret = my.createYearGenderTables();

	ret = my.execAllYearGenderQuery(&MySql::addTagCountInYearGenderTables);
	ret = my.execAllYearGenderQuery(&MySql::updateTagCountInYearGenderTables);
	ret = my.execAllYearGenderQuery(&MySql::createYearGenderRecResultTables);
	ret = my.execAllYearGenderQuery(&MySql::insertYearGenderRecResultTables);

	ret = my.execAllYearGenderQuery(&MySql::createUserKeywordYearGenderTables);

	ret = my.execAllYearGenderQuery(&MySql::insertUserKeywordYearGenderTables);

	ret = my.execAllYearGenderQuery(&MySql::createYearGenderItemKeywordTables);

	ret = my.execAllYearGenderQuery(&MySql::insertYearGenderItemKeywordTables);

	ret = my.execAllYearGenderQuery(&MySql::createYearGenderOriginalRecResultTables);
	ret = my.execAllYearGenderQuery(&MySql::insertYearGenderOriginalRecResultTables);

	ret = my.execAllYearGenderQuery(&MySql::createYearGenderUserItem);
	ret = my.execAllYearGenderQuery(&MySql::insertYearGenderUserItem);



    ret = my.execAllYearGenderQuery(&MySql::createYearGenderLeaderBoardTables);
    ret = my.execAllYearGenderQuery(&MySql::createYearGenderFinalEvaluationTables);
    ret = my.execAllYearGenderQuery(&MySql::evaluateAlgorithmUserItemKeyword);
    ret = my.execAllYearGenderQuery(&MySql::trainingAlgorithmUserItemKeyword);


    //int index = sizeof(MySql::statements)/sizeof(char*) - 1;
    //ret = my.execQuery(MySql::statements[index]);

    ret = my.execAllYearGenderQuery(&MySql::createYearGenderUserSnsFollowerTables);
    */
    ret = my.execAllYearGenderQuery(&MySql::createYearGenderUserSnsUserItemTables);
    //ret = my.query();
    //myFileTool();



#else
    QueryThread query;
    /*
    if (query.exec(//"drop table if exists user_profile;"
		//"create table user_profile(UserId integer primary key, YearOfBirth integer not null, Gender integer not null, NumberOfTweet integer not null, TagIds text not null);"
		"load data local infile '/home/nick/download/track1/user_profile.txt' into table user_profile;"))
    {
        query.join();
    }
    */
    int i = 1990, j = 1;
    char queryStr[1024];
    string str;
    char dstTableName[128];
	char srcTableName[128];
	sprintf(dstTableName, "YearGenderItemKeyword%d%c", i, gender[j]);
	sprintf(srcTableName, "UserKeywordYearGender%d%c", i, gender[j]);
	sprintf(queryStr, "drop table if exists %s;", dstTableName);
	str = queryStr;
	sprintf(queryStr, "create table %s(UserId integer not null, ItemId integer not null, Keyword integer not null, Weight real not null, primary key(UserId, ItemId, Keyword));", dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_UserId_index(UserId);", dstTableName, dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_ItemId_index(ItemId);", dstTableName, dstTableName);
	str += queryStr;
	sprintf(queryStr, "alter table %s add index %s_Keyword_index(Keyword);", dstTableName, dstTableName);
	str += queryStr;

	sprintf(queryStr, "insert into %s(UserId, ItemId, Keyword, Weight) select s.UserId, i.ItemId,"
         " s.Keyword, s.Weight*i.Weight as Weight from %s s, item_detail_weight i where "
         "s.Keyword=i.Keyword and s.UserId%100=0;",
			dstTableName, srcTableName);
	str += queryStr;

    if (query.exec(str))
    {
        query.join();
    }

#endif
    startTime = (time(NULL) - startTime);
    int min = startTime/60;
    int sec = startTime%60;
    cout << "total time is " << min << "min " << sec << "sec" <<endl;
    return 0;
}

	//ret = my.execAllYearGenderQuery(&MySql::createAlgorithmUserItemKeyword);
	//ret = my.execAllYearGenderQuery(&MySql::insertAlgorithmUserItemKeyword);


